/***********************************************************************/
/*                                                                     */
/*               Output Analysis Program for ETMSP (OAP)               */
/*               =======================================               */
/*                                                                     */
/*        copyright @(1993) Electric Power Research Institute          */
/*                                                                     */
/* EPRI reserves all rights in the software and associated             */
/* documentation as delivered. The software and its associated         */
/* documention or any portion thereof may not be reproduced in any form*/
/* whatsoever without the consent of EPRI, except as explicity provided*/
/* for by license. Licensee shall submit a detailed description to EPRI*/
/* of any changes or modifications to the software within two (2)      */
/* months after such changes or modifications are made. At EPRI's sole */
/* discretion, the software, as changed or modified, shall be given a  */
/* new designation sufficiently different from its then current        */
/* designation as to prevent mistake,confusion, or deception as between*/
/* the current software as changed or modified.                        */
/*                                                                     */
/* A license under EPRI's rights in the software can be obtained       */
/* directly from EPRI.                                                 */
/*                                                                     */
/* This program and supporting materials may be independently available*/
/* from Electric Power Software centre (EPSC) for an appropriate       */
/* distribution fee. The EPSC is a service of                          */
/*                                                                     */
/*                    Power Computing Company                          */
/*                    1930 Hi Line                                     */
/*                    Dallas, Tx 75207                                 */
/*                                                                     */
/* NEITHER EPRI, ANY MEMBER OF EPRI, NOR ANY PERSON OR  ORGANIZATION   */
/* ACTING ON BEHAVE OF THEM:                                           */
/*                                                                     */
/* 1. MAKE ANY WARRANTY OR REPRESENTATION WHATSOEVER, EXPRESS OR       */
/*    IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY OR FITNESS OR */
/*    ANY PURPOSE WITH RESPECT TO THE SOFTWARE OR ITS ASSOCIATED       */
/*    DOCUMENTATION; OR                                                */
/*                                                                     */
/* 2. ASSUMES ANY LIABILITY WHATSOEVER WITH RESPECT TO ANY USE OF THE  */
/*    SOFTWARE OR ITS ASSOCIATED DOCUMENTATION OR ANY PORTION THEREOF  */
/*    OR WITH RESPECT TO ANY DAMAGES WHICH MAY RESULT FROM SUCH USE.   */
/*                                                                     */
/* THIS NOTICE MAY NOT BE REMOVED FROM THE SOFTWARE BY ANY USER        */
/* THEREOF.                                                            */
/*                                                                     */
/***********************************************************************/

/***********************************************************
 *
 * File:          x2d.c
 * Description:   plot  OAP outputs using X11 library 
 *                routines 
 *
 ***********************************************************/

/*
 *    Include File
 */

#include "PlotX-PS.h"

#ifdef VMS
#include <Xlib.h>
#include <Xutil.h>
#include <Xos.h>
#else
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#endif

#include "epri.h"
#include "hydro.h"
#include "fl.h"

/*
 *    Function Declarations
 */
/* 
int textlen();
xp_label_();
refresh_();
clear_w_();
initg_();
initwindow_();
xp_vertical();
xp_box_();
plot_Xxticklabel();
plot_Xyticklabel();
xp_ticks();
int line_type();
xplot_()
line_draw();
marker_draw();
get_time();
paint_b();
xp_end_();
*/

/*
 *    Preprocessor         
 */

/* window size and location */
#define X_ORG            5
#define Y_ORG            5
#define X_DIM            900
#define Y_DIM            700

#define DEFAULT_XBDR_DIM 80
#define DEFAULT_YBDR_DIM 150
#define DEFAULT_BORDER_WIDTH  3

/*
 *    Global Variables
 */

Time LastEventTime;
GC  gc;                                 /* graphics context */
Display     *display;                   /* display */
int         screen;                     /* screen */
Window      window,                     /* parent window */
            bt1, bt2;
XFontStruct *font_info;                 /* font information */
XEvent   event;
Pixmap charmap, hydro_pixmap, pixmap;
int foreground,background;
int win_w, win_h;
int ylabel_position[30];
int Xxmin, Xymin, Xxmax, Xymax;
int depth;
char *color_name[10];
unsigned long line_colors[10];

/***********************************************************/
int textlen(str, maxlen)
char *str;
int maxlen;
{
   char *cpa;
   int ia;
   
   *(str+maxlen-1)=0;
   ia = strlen(str);
   for (; ia>1; ia--) {
     cpa = str +ia -1;
     if (*cpa !=' ') return ia; }
   return 1;
}
/***********************************************************/
void getini_()
{
   FILE *ifpt;
   int l;
   char ini_file[80], inline[150];
   char *cpa, *word1, *word2, *spac1, *spac2;
   
/* assign defaults for fonts & printers */
   strcpy(startup_.defont, DEFAULT_FONT);
   strcpy(startup_.lgfont, LOGO_FONT);
   strcpy(startup_.prtnam[0], PRNTER1);
   strcpy(startup_.prtnam[1], PRNTER2);
   strcpy(startup_.prtnam[2], PRNTER3);

   l = textlen (startup_.homedir,80);
   strncpy(ini_file,startup_.homedir,l);
   ini_file[l]='\0';
#ifdef VMS
   strcat(ini_file,":ITOAP.INI");
#else
   strcat(ini_file,"/ITOAP.INI");
#endif
   if ((ifpt = fopen(ini_file,"r")) == NULL) {
    fprintf(stderr,
    "\t\tITOAP user init file \"%s\"\n\
\t\tnot found. Default value will be used.\n",
      ini_file);
    return;
   }
   else {
     do {
       cpa = fgets (inline, 150, ifpt); 
       if (!cpa) break;
       for (cpa = strchr(inline,0) -1; ; cpa--){
         if(!strchr(" \n\t", *cpa)) {
          cpa++; *cpa=0; break;}
       }
       word1=inline+strspn(inline," ");
       if (! *word1) continue;
       spac1 = word1 + strcspn (word1, " \t");
       word2 = spac1 + strspn (spac1, " \t");
       if (! *word2) continue;
       spac2 = word2 + strcspn (word2, " \t\n");
       *spac1 = 0; *spac2 = 0; 
       if ( !strcmp(word1,"logo_font")) 
          strcpy(startup_.lgfont,word2); 
       if ( !strcmp(word1,"default_font")) 
          strcpy(startup_.defont,word2); 
       if ( !strcmp(word1,"postscript_printer_1")) 
          strcpy(startup_.prtnam[0],word2); 
       if ( !strcmp(word1,"postscript_printer_2")) 
          strcpy(startup_.prtnam[1],word2); 
       if ( !strcmp(word1,"postscript_printer_3")) 
          strcpy(startup_.prtnam[2],word2);  
     } while(!feof(ifpt)); 
     fclose(ifpt);
   } 
}
/***********************************************************
 * write plot title and y axis labels
 */
xp_label_(ip,lentl,lend,lenyl,
   lenl1,lenl2,ym,ya,iflg)
float *ym,*ya;
int *ip,*lentl,*lend,*lenyl,*lenl1,*lenl2,*iflg;
{
   char ylabel[80], yl1[80], yl2[80], tl[80], time[100];
   int    font_height, font_width;
   int    text_len, text_xpos, text_ypos;
   int  i, j, ipm1;
   static int nmfile;
   extern title_index;
   float ytmp, dy;

   /* get current time */
   get_time(time);

   ipm1 = *ip-1;

   strncpy(tl, ptit_.ctitl2[ipm1], *lentl);
   strncpy(ylabel, ptit_.ytitle[ipm1], *lenyl);
   strncpy(yl1, ptit_.ltitle[ipm1], *lenl1);
   strncpy(yl2, ptit_.ltitl2[ipm1], *lenl2);

   tl[*lentl] = '\0';
   title_index = *lentl;
   ylabel[*lenyl] = '\0';
   yl1[*lenl1] = '\0';
   yl2[*lenl2] = '\0';
   ymic= *ym;
   ymac= *ya;

   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* write title  */
    if (*ip == 1) {
        nmfile = 0;

        for(i=1; i<xychnl_.kch1; i++)
            nmfile += memcmp(&(ptit_.ctitl2[i-1][0]),
                &(ptit_.ctitl2[i][0]), 80) && ptit_.ctitl2[i][0] != ' '
                ? 1 : 0;

        j = 6;

        while(j && strspn(ptit_.ptitle[--j], " "));

        j++;
        /* write current time */
        text_len   = strlen(time);
        text_xpos =  Xxmax - XTextWidth(font_info,time,text_len) - 10;
        text_ypos = Xymin - (j+8)*font_height/2;
        XDrawString(display,pixmap, gc,text_xpos, text_ypos,time,text_len);

        if(!nmfile) {
            XDrawString(display,pixmap,gc,Xxmin,text_ypos,ptit_.caseid[1],12);
        }
        text_len = 80;
        for(i=0; i<j; i++) {
            text_ypos = Xymin - (j-2*i+6)*font_height/2;
            ptit_.ptitle[i][79] = '\0';
            XDrawString(display,pixmap,gc,Xxmin,text_ypos,ptit_.ptitle[i],
                text_len);
        }
    }

   /* using vertical font to write y axis labels (three parts)  */
   /* skip if iflg=2 and ip=1 */
    if(*iflg != 2 || *ip == 1) {	
        text_xpos  = Xxmin - *ip*50;
/*        text_ypos  = Xymax+28;*/
        text_ypos  = Xymax;
        text_len   = *lenyl;

        if(status_.sscale) 
        {
            if(*ip == 1) {
                xp_vertical(ylabel,text_xpos,Xymax-50,text_len);

                if(!nmfile) {
                    text_len   = *lentl;
                    xp_vertical(tl,Xxmin-70,Xymax-8,text_len);
                }
            }

            if(nmfile) {
                text_len = *lentl;
                xp_vertical(tl,Xxmin-((*ip+1)/2)*40-45,
                    Xymax-8-((*ip+1)%2)*(Xymax-Xymin)/2,text_len);
                xp_vertical(ptit_.caseid[ipm1], Xxmin-((*ip+1)/2)*40-45,
                    Xymax-130-((*ip+1)%2)*(Xymax-Xymin)/2,12);
            }

            text_len   = *lenl1;
            xp_vertical(yl1,Xxmin-((*ip+1)/2)*40-60,
                Xymax-8-((*ip+1)%2)*(Xymax-Xymin)/2,text_len);
        }
        else {
            xp_vertical(ylabel,text_xpos-3,text_ypos,text_len);
            if(nmfile || *ip == 1)
                xp_vertical(tl,text_xpos+12,text_ypos,*lentl);
            if(nmfile) xp_vertical(ptit_.caseid[ipm1],
                text_xpos+12,text_ypos-160,12);
            text_len = *lenl1;
            xp_vertical(yl1,text_xpos-3,text_ypos-200,text_len);
            text_len = *lenl2;
            xp_vertical(yl2,text_xpos+12,text_ypos-350,text_len);
        }
        if(*iflg == 9 && *ip == xychnl_.kch1) {
            xp_vertical("LEAST SQUARE FIT RESULT",text_xpos-18,
                text_ypos-135,23);
        }
   
   /* write y axis annotations */
        if(!status_.sscale || *ip == 1)
        {	
            dy = (ymac-ymic)/num_ylabel;
            for (i=0; i<=num_ylabel; i++){
                ytmp = ymic + i*dy;
                plot_Xyticklabel(ip, ytmp,ylabel_position[i]);
            }
        }
    }
}
/***********************************************************
 * refresh plotting area   
 */
void refresh_()
{
   XCopyArea(display, pixmap, window, gc, 0, 0, win_w, win_h, 0, 0);
}
/***********************************************************
 * clear window  
 */
clear_w_()
{
   int Xwid,Ywid;
   XFlush(display);
   XSetForeground(display, gc,  WhitePixel(display,screen)); 
   XFillRectangle(display,  pixmap, gc, 0, 0, win_w, win_h );   
   if (!(depth==1)) {
   XSetForeground(display,gc,line_colors[8]);
   Xwid  = win_w  - 5*DEFAULT_XBDR_DIM;
   Ywid  = win_h   - 2*DEFAULT_YBDR_DIM;
   XFillRectangle(display,  pixmap, gc, Xxmin,Xymin,Xwid,Ywid);
   }
   XSetForeground(display, gc, BlackPixel(display,screen));  
   LastEventTime = CurrentTime;
}
/***********************************************************
 * graph parameter setting 
 */
initg_(n1,n2,xm,xa,xtk,ytk,n_x,n_y)
int *n1,*n2,*xtk,*ytk,*n_x,*n_y;
float *xm,*xa;
{
   extern char xlabel[80];
   extern int xticks, yticks,num_ylabel;
   extern double  deltx, delty, xmic, xmac;
   strncpy(xlabel,ptit_.ftitle,*n1);
   xlabel[*n1]='\0';
   xmic= *xm;
   xmac= *xa;
   xticks= *xtk;
   yticks= *ytk;
   num_ylabel= *n_y;
   num_xlabel= *n_x;
   delty = 10./(float)num_ylabel;
   deltx = (xmac-xmic)/(float)num_xlabel;
}
/***********************************************************
 * create top-level X window of the application  
 * create two button subwindows
 * set basic window parameters and hints
 */
initwindow_()
{
   XGCValues  gcvalues;
   char default_geometry[100];
   int i;
   int  x, y;
   int border_width;
   int white_bg;
   char *display_name = NULL;
   char *geometry;
   char *window_name = "ITOAP"; 
   char *icon_name = "OAP";
   char    *option;
   XSizeHints  size_hints;
   XSetWindowAttributes attrib;
   Colormap col_map;
   XColor scdef;

   attrib.backing_store = WhenMapped;

   /* Open display , get screen info */
   if (!(display = XOpenDisplay(display_name))) {
      fprintf(stderr,"Could not open display %s\n",
              display_name);
      exit(1);
   }
   screen = DefaultScreen(display);

   sprintf(default_geometry,"=%dx%d+%d+%d",X_DIM,Y_DIM,X_ORG,Y_ORG);
   foreground = BlackPixel(display,screen);
   white_bg = WhitePixel(display,screen);
   attrib.background_pixel = white_bg;
   border_width = DEFAULT_BORDER_WIDTH;

   depth = DisplayPlanes(display,screen);
   col_map = DefaultColormap(display,screen);
   if (depth == 1)
   {
      for (i=0; i<10; i++)
      {
        line_colors[i]= foreground;
      }
      background = white_bg;
   }
   else
   {
      color_name[0] = "black";
      color_name[1] = "red";
      color_name[2] = "green3";
      color_name[3] = "magenta1";
      color_name[4] = "purple3";
  /*  color_name[4] = "deep sky blue"; */
      color_name[5] = "chocolate1";
      /* color_name[6] = "turquoise1"; */
      color_name[6] = "dark olive green";
      color_name[7] = "spring green"; 
      color_name[8] = "LightSkyBlue1"; 
      color_name[9] = "cyan"; 
      for (i=0; i<10; i++)
      {
        if (XAllocNamedColor(display, col_map,
            color_name[i], &scdef, &scdef))
          line_colors[i]= scdef.pixel;
        else {
          fprintf(stderr,"Can't allocate color %s\n",
           color_name[i]);
          exit(1);
        }
      }
      background = line_colors[7];
   } 
   /* initialize  window size and location */
   geometry="=+5+5";                  
   if (!XGeometry(display,screen,geometry,
	default_geometry,border_width,
        0,0,0,0,&x,&y,&win_w,&win_h)) 
     {
      x = X_ORG;
      y = Y_ORG;
      win_w = X_DIM;
      win_h = Y_DIM;
     }   

   /* create the window */
   window = XCreateWindow(display,RootWindow(display,screen),
          x, y, win_w, win_h,border_width,
          DefaultDepth(display,screen), 
          InputOutput,CopyFromParent,
          CWBackingStore, &attrib);

   /* create two button subwindows */
   bt1 = XCreateWindow(display,window,
          20, win_h-60, 64, 20, 2,
          DefaultDepth(display,screen), 
	  InputOutput,CopyFromParent,
          CWBackingStore, &attrib);
   bt2 = XCreateWindow(display,window,
          110, win_h-60, 64, 20, 2,
          DefaultDepth(display,screen), 
	  InputOutput,CopyFromParent,
          CWBackingStore, &attrib);

   /* initialize size hint property for window manager */
   size_hints.flags = PPosition | PSize | PMinSize;
   size_hints.x = x;
   size_hints.y = y;
   size_hints.width = win_w;
   size_hints.height = win_h;
   size_hints.min_width = X_DIM;
   size_hints.min_height = Y_DIM;

   /* creates a pixmap for vertical character font set */
   charmap = XCreatePixmapFromBitmapData(display, window,f_bits,
          f_width, f_height,foreground,white_bg,
	  DefaultDepth(display, screen));

   /* creates a pixmap of "Ontario Hydro"or "EPRI" as logo and 
      icon pixmap */
#ifdef _LOGO_OH
   hydro_pixmap = XCreatePixmapFromBitmapData(display,window,
          hydro_bits, hydro_width, hydro_height,foreground,
          white_bg,DefaultDepth(display, screen));
#endif
#ifdef _LOGO_EPRI
   hydro_pixmap = XCreatePixmapFromBitmapData(display,window, 
	  epri_bits, epri_width, epri_height,foreground,
  	  white_bg,DefaultDepth(display, screen));
#endif

   /* set standard properties of the window */
   XSetStandardProperties(display,window, window_name, icon_name,
          hydro_pixmap, NULL, 0, &size_hints);

   /* select the event type to be sent to the top-level window
      and subwindows */
   XSelectInput(display,window, ExposureMask |
      FocusChangeMask | KeyPressMask | StructureNotifyMask );
   XSelectInput(display,bt1, ButtonPressMask |ButtonReleaseMask |
        ExposureMask);
   XSelectInput(display,bt2, ButtonPressMask |ButtonReleaseMask |
        ExposureMask);

   /* Create GC for text and drawing */
   gc = XCreateGC(display, window, 0, &gcvalues);

   /* specify foreground */
   XSetForeground(display, gc, foreground);

   /* creates a pixmap for drawing area */
   pixmap = XCreatePixmap(display, window, win_w, win_h,
           DefaultDepth(display, screen));

   Xxmin = 4*DEFAULT_XBDR_DIM;
   Xymin = DEFAULT_YBDR_DIM;

   /* display the window */
   XMapSubwindows(display,window);
   XMapWindow(display, window);
}
/***********************************************************
 * window event handling 
 */
event_handling_()
{
   XComposeStatus compose;
   char     buffer[1];
   int      bufsize = 1;
   /* int Width,Height; */
   XSizeHints  size_hints;

   Window child;
   Window root;
   int win_x, win_y,x,y;
   unsigned int button;

   /* loop to get events on the main window and subwindows */
   while(1) {
    XNextEvent(display, &event);

    switch (event.type) {

   /* if expose, refresh the plot and paint button */
      case Expose:
   /* get rid of all other Expose events on the queue */
        while (XCheckTypedEvent(display, Expose, &event));
	refresh_();
	paint_b();
        break;

   /* if ButtonPress occurs on the button window, do
      the specified command */
      case ButtonPress:
        if(event.xbutton.window ==bt1 || event.xbutton.window ==bt2)
	{
	 if (event.xbutton.time <= LastEventTime) break;
         button = event.xbutton.button;
   /* get the matching ButtonRelease on same button */
         while (1) {
             while (XCheckTypedEvent(display,ButtonPress,&event));
              XMaskEvent(display, ButtonReleaseMask, &event);
               if (event.xbutton.button == button) {
                  paint_b();
                  break; }
         }
         if(event.xbutton.window ==bt1) {
                return;
                break; }
         if(event.xbutton.window ==bt2) {
    		XRaiseWindow(display,window);
             	refresh_();
                break; }
        }
        break;

    /* window has been resized,change width and height 
      case ConfigureNotify:
         Width  = event.xconfigure.width;
         Height = event.xconfigure.height;
         break;
    */
      default:
         break;
    }
  }
}
/***********************************************************
 * write vertical label using vertical font set  
 */
xp_vertical(text,text_xpos,text_ypos,text_len)
int text_xpos,text_ypos,text_len;
char text[80];
{
   char c;
   int i, as;

   for (i=0; i<text_len; i++) {
      c=text[i];
      as = toascii(c);
      XCopyArea (display, charmap, pixmap, gc, 0, as*6,
          13, 6, text_xpos, text_ypos-i*6);
   }
}
/***********************************************************
 * draw x and y axes; 
 * write x title and current time;
 * draw Logo and write plot title;
 * call xp_ticks to draw x and y axes ticks, 
 * x axis annotation and grids  
 */
xp_box_()
{
   XRectangle     rectangles[1];
   char     *font_name;
   int    Xwid, Ywid;
   extern int    Xxmin, Xymin, Xxmax, Xymax;
   int    font_height;
   int    text_len, text_width, text_xpos, text_ypos;

   Xwid  = win_w  - 5*DEFAULT_XBDR_DIM;
   Ywid  = win_h   - 2*DEFAULT_YBDR_DIM;
   Xxmax = Xxmin + Xwid;
   Xymax = Xymin + Ywid;

   /* Draw the main axes */
   XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
   XDrawRectangle(display,pixmap,gc,Xxmin,Xymin,Xwid,Ywid);

   /* logo */
   XCopyArea(display, hydro_pixmap, pixmap, gc,
             0, 0, 96, 48, 20,20);

   /* Find a font and load it */
   font_name = startup_.lgfont;
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"Could not open font %s\n",font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;

   /* part of Hydro logo */
#ifdef _LOGO_OH
   XDrawString(display,pixmap, gc, 80, 55, "Ontario Hydro", 13);
#endif

   /* part of EPRI logo */
#ifdef _LOGO_EPRI
   XDrawString(display,pixmap, gc, 10, 55,
     "Leadership in Science and Technology", 37);
#endif

   /* X-Axis Label */
   text_len   = strlen(xlabel);
   text_width = XTextWidth(font_info,xlabel,text_len);
   text_xpos  = (Xxmax - text_width)/2;
   text_ypos  = Xymax + 5*font_height;

   XDrawString(display,pixmap,gc,text_xpos,text_ypos,xlabel,text_len); 

   /* Plot title */
   text_ypos  = Xymax + 3*font_height;

   if(!status_.sscale) 
  /* Find a font and load it */
   font_name = startup_.defont;
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"Could not open font %s\n",font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);

    /* text_len = lens_(&(ptit_.xtitle[0]),80); */
    text_len = textlen (&(ptit_.xtitle[0]),80);
    
    text_width = XTextWidth(font_info,ptit_.xtitle,text_len);
    text_xpos =  (Xxmin + Xxmax - text_width)/2;
    XDrawString(display, pixmap, gc, text_xpos, text_ypos,
        ptit_.xtitle, text_len);

   /*Draw the ticks, annotations and grids  */
   xp_ticks();
   
}
/***********************************************************
 * draw annotations on the x-axis 
 */
plot_Xxticklabel(annotation,Xxtmp,font_height)
double annotation;
int    Xxtmp, font_height;
{
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[30];

   sprintf(text,"%.3g",annotation);
   text_len   = strlen(text);
   text_width = XTextWidth(font_info,text,text_len);
   text_xpos  = Xxtmp  - text_width/2;
   text_ypos  = Xymax + 2*font_height;
   /* XDrawString(display,window,gc,text_xpos,
	text_ypos,text,text_len); */
   XDrawString(display,pixmap,gc,text_xpos,
	text_ypos,text,text_len);
}

/***********************************************************
 * draw annotations on the y-axis
 */
plot_Xyticklabel(ip, annotation, Xytmp)
double annotation;
int    *ip, Xytmp;
{
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[30];

   sprintf(text,"%6.2f",annotation);
   text_len   = strlen(text);
   text_width = text_len*6;
/*
   text_xpos  = Xxmin - *ip *40 +17;
*/
   text_xpos  = Xxmin - *ip *50 +27;
   text_ypos  = Xytmp  + text_width/2;
   xp_vertical(text,text_xpos,text_ypos,text_len);
}
/***********************************************************
 * draw minor nad major ticks on x and y axes;
 * draw x annotations;
 * draw grids
 */
xp_ticks()
{
   extern int     xticks,yticks;
   extern double  deltx, delty, ymic, ymac;

   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   double ymx;
   int    Xxtmp, Xytmp;
   int    font_height;
   int    i, grid_flag;
   int    grid;

   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;

   ymx = num_ylabel*delty;
   xfactor = (Xxmax - Xxmin)/(xmac-xmic);
   yfactor = (Xymin - Xymax)/ymx;
   
   /* xticks */

   /* draw minor tick marks */
   xtmp = xmic;
   while (xtmp < xmac) {
      for (i=1; i<xticks; i++) {
         xt = xtmp + i*deltx/xticks;
         if ((xt > xmic) && (xt < xmac)) {
            Xxtmp = Xxmin + (xt-xmic)*xfactor;
            if (i==xticks/2) {
               XDrawLine(display,pixmap,gc,Xxtmp,Xymin,Xxtmp,Xymin+7);
               XDrawLine(display,pixmap,gc,Xxtmp,Xymax,Xxtmp,Xymax-7);
            } else {
               XDrawLine(display,pixmap,gc,Xxtmp,Xymin,Xxtmp,Xymin+4);
               XDrawLine(display,pixmap,gc,Xxtmp,Xymax,Xxtmp,Xymax-4);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks on x axis and accompanying annotations */
   for (i=0; i<=num_xlabel; i++) {
      xtmp = i*deltx;
      Xxtmp = Xxmin + xtmp*xfactor;
      XDrawLine(display,pixmap,gc,Xxtmp,Xymin,Xxtmp,Xymin+10);
      XDrawLine(display,pixmap,gc,Xxtmp,Xymax,Xxtmp,Xymax-10);
      xtmp = xmic + xtmp;
      plot_Xxticklabel(xtmp,Xxtmp,font_height); 
   }

   /* yticks */

   /* draw minor tick marks */
   ytmp = 0.;
   while (ytmp < ymx) {
      for (i=1; i<yticks; i++) {
         yt = ytmp + i*delty/yticks;
         if ((yt > 0.) && (yt < ymx)) {
            Xytmp = Xymax + yt*yfactor;
            if (i==yticks/2) {
               XDrawLine(display,pixmap,gc,Xxmin,Xytmp,Xxmin+7,Xytmp);
               XDrawLine(display,pixmap,gc,Xxmax,Xytmp,Xxmax-7,Xytmp);
            } else {
               XDrawLine(display,pixmap,gc,Xxmin,Xytmp,Xxmin+4,Xytmp);
               XDrawLine(display,pixmap,gc,Xxmax,Xytmp,Xxmax-4,Xytmp);
            }
         }
      }
      ytmp += delty;
   }

   ylabel_position[0] = Xymax;
   /* major tickmarks */
   for (i=1; i<num_ylabel; i++) {
      ytmp = i*delty;
      Xytmp = Xymax + ytmp*yfactor;
      XDrawLine(display,pixmap,gc,Xxmin,Xytmp,Xxmin+10,Xytmp);
      XDrawLine(display,pixmap,gc,Xxmax,Xytmp,Xxmax-10,Xytmp);
      ylabel_position[i]=Xytmp;
   }
   ylabel_position[num_ylabel] = Xymin;

   /* grids on x and y directions */
   /* set grid flag ON as default */
   grid_flag = ON;
   if (grid_flag == ON) {
      grid = line_type(11);
      xtmp = xmic + deltx;
      while (xtmp < xmac) { 
         Xxtmp = Xxmin + (xtmp-xmic)*xfactor;
         XDrawLine(display,pixmap,gc,Xxtmp,Xymin+10,Xxtmp,Xymax-10);
         xtmp += deltx;
      }
      ytmp =  delty;
      while (ytmp < ymx) { 
         Xytmp = Xymax + ytmp*yfactor;
         XDrawLine(display,pixmap,gc,Xxmin+10,Xytmp,Xxmax-10,Xytmp);
         ytmp += delty;
      }
      XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
   }
}
/***********************************************************
 * set line color
 */
set_line_color(id)
int id;
{
   id = id % 8;
   switch (id)
   {
      case 0 : XSetForeground(display,gc,line_colors[0]);
               break;
      case 1 : XSetForeground(display,gc,line_colors[1]);
               break;
      case 2 : XSetForeground(display,gc,line_colors[2]);
               break;
      case 3 : XSetForeground(display,gc,line_colors[3]);
               break;
      case 4 : XSetForeground(display,gc,line_colors[4]);
               break;
      case 5 : XSetForeground(display,gc,line_colors[5]);
               break;
      case 6 : XSetForeground(display,gc,line_colors[6]);
               break;
      case 7 : XSetForeground(display,gc,line_colors[7]);
               break;
      default: XSetForeground(display,gc, foreground);
   }
}
/***********************************************************
 * set the line type
 * 	0. points
 * 	1. solid line
 *	2. dashed line
 *	3. long dashed line
 *	4. dot dashed line
 *	5. short dotted line
 *	6. short dashed line
 *	7. odd dashed line
 *	8. dotted line
 *	9. dash dotted line
 *     10. equal dashed line
 *     11. long dotted line
 */
int line_type(linetyp)
int linetyp;
{
   static unsigned char dashed[2]  = {4,4};
   static unsigned char dotted[2]  = {1,6};
   static unsigned char dot_dashed[4] = {3,4,3,1};
   static unsigned char short_dotted[2] = {1,3};
   static unsigned char short_dashed[2] = {3,5};
   static unsigned char odd_dashed[3]  = {1,2,3};
   static unsigned char long_dashed[2] = {7,4};
   static unsigned char dash_dotted[4]   = {4,3,1,3};
   static unsigned char equal_dashed[2] = {6,6};
   static unsigned char long_dotted[2]   = {1,8};
   int      linepat = ON;

   switch (linetyp) {
   case 0 : XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
            linepat = OFF;
            break;
   case 1 : XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
            break;
   case 2 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dashed,2);
            break;
   case 3 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,long_dashed,2);
            break;
   case 4 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dot_dashed,4);
            break;
   case 5 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,short_dotted,2);
            break;
   case 6 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,short_dashed,2);
            break;
   case 7 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,odd_dashed,3);
            break;
   case 8 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dotted,2);
            break;
   case 9 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_dotted,4);
            break;
   case 10: XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,equal_dashed,2);
            break;
   case 11: XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,long_dotted,2);
   }
   return(linepat);
}
/***********************************************************
 * draw curve and legend
 */
xplot_(ip,start_p,end_p,xtem,ytem,kkl,marker_flag,kkm,iflg)
int *ip, *start_p, *end_p, *kkl,*kkm,*marker_flag,*iflg;
float xtem[], ytem[];
{
   extern double  xmic, xmac, ymic, ymac;
   extern int  Xxmin, Xymin, Xxmax, Xymax;
   extern int title_index;
   int    font_height, xpos, ypos;
   double  idx,jdy;
   int     j,kl,km;
   int  maxpt;
   int  jk;
   XPoint  points[3000];

   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;

   kl= *kkl;
   km = *kkm;
   idx = (Xxmax-Xxmin)/(xmac-xmic);
   jdy = (Xymax-Xymin)/(ymic-ymac);
   maxpt = *end_p - *start_p+1;
   for (j=0; j<maxpt; j++) 
   {
       points[j].x = (int)((xtem[*start_p-1+j]-xmic)*idx) + Xxmin;
       points[j].y = (int)((ytem[*start_p-1+j]-ymac)*jdy) + Xymin;
   }

   /* plot the curve */
   line_draw(points,&maxpt,kl);

   /* plot the markers */
   if (*marker_flag==ON)
      marker_draw(points, &maxpt,km);

   if(!status_.sscale) {

   if(*iflg != 2 || *ip == 1) {
   /* plot legend along vertical direction */
	ypos  = Xymin;
	xpos  = Xxmin + 5 - *ip*50;
	points[0].x = xpos;
	points[0].y = ypos;
	points[1].x = xpos;
	points[1].y = ypos + 50;
    }
    }
    else {
	ypos  = 0.62*Xymax - ((*ip+1)%2)*(Xymax-Xymin)/2 + 0.38* Xymin;
	points[0].x = points[1].x = Xxmin - ((*ip+1)/2)*40 - 54;
	points[0].y = ypos;
	points[1].y = ypos - 0.09*(Xymax - Xymin);
    }
	jk = 2;
	line_draw(points,&jk,kl);

	if (*marker_flag==ON)  
		marker_draw(points, &jk, km);
}

/***********************************************************
 * draw the curve using selected line type 
 */
line_draw(points,maxpt,kl)
XPoint points[];
int    *maxpt, kl;
{
   int line_type();
   int line_on;

   if (depth==1) {
     line_on = line_type(kl);
   }
   else {
     line_on = line_type(1);
   }
   set_line_color(kl);
   if (line_on)
   {
   /* XDrawLines(display,window,gc,points,*maxpt,CoordModeOrigin); */
      XDrawLines(display,pixmap,gc,points,*maxpt,CoordModeOrigin);
   }
   else      
   {
   /* XDrawPoints(display,window,gc,points,*maxpt,CoordModeOrigin); */
      XDrawPoints(display,pixmap,gc,points,*maxpt,CoordModeOrigin);
   }
   set_line_color(0);
}
/***********************************************************
 * draw the markers
 * 1. circle
 * 2. cross
 * 3. square
 */
marker_draw(points,maxpt,km)
XPoint points[];
int    *maxpt, km;
{
   int    j;
   int count,dmark;

   XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
   dmark = (int)((float)*maxpt/36.);
   if (dmark == 0) dmark=1;
   switch (km) {
     case 0 : 
     /*  XDrawPoints(display,window,gc,points,*maxpt,CoordModeOrigin); */
         XDrawPoints(display,pixmap,gc,points,*maxpt,CoordModeOrigin);
         break;
     case 1 : 
         j=0;
         count = 1;
         while (j <*maxpt-2) {
            j = dmark * count;
         /* XDrawArc(display,window,gc,points[j].x-2,
		points[j].y-3,6,6,0,360*64); */
	    XDrawArc(display,pixmap,gc,points[j].x-2,
		points[j].y-3,6,6,0,360*64);
            count++;
         }
         break;
     case 2 : 
         j=0;
         count = 1;
	 while (j <*maxpt-2) {
            j = dmark * count;
         /* XDrawLine(display,window,gc,points[j].x-2,points[j].y-2,
                points[j].x+2,points[j].y+2); */
            XDrawLine(display,pixmap,gc,points[j].x-2,points[j].y-2,
                points[j].x+2,points[j].y+2);
         /* XDrawLine(display,window,gc,points[j].x+2,points[j].y-2,
                points[j].x-2,points[j].y+2); */
            XDrawLine(display,pixmap,gc,points[j].x+2,points[j].y-2,
                points[j].x-2,points[j].y+2);
	    count++;
         }
         break;
     case 3 : 
         j=0;
         count = 1;
         while (j <*maxpt-2) {
            j = dmark * count;
         /* XDrawRectangle(display,window,gc,points[j].x-2,
		points[j].y-2,4,4); */
            XDrawRectangle(display,pixmap,gc,points[j].x-2,
		points[j].y-2,4,4);
            count++;
         }
         break;
   }
}
/***********************************************************
 * get current time 
 */
get_time(newtime)
char newtime[];
{
   char      *ctime();
   time_t    time();

   int       i;
   time_t    timeofday;
   char      *loctime;

   /* get the local time */
   timeofday = time((time_t*)0);
   loctime   = ctime(&timeofday);
   newtime[0] = 'D';
   newtime[1] = 'a';
   newtime[2] = 't';
   newtime[3] = 'e';
   newtime[4] = ' ';
  /* loctime has a '\n' in it, so delete that character */
   i=0;
   while (loctime[i] != '\n') {
      newtime[i+5] = loctime[i+4];
      i++;
   }
   newtime[i+1] = '\0';
}
/***********************************************************
 * draw labels on button subwindows 
 */
paint_b()
{
   XSetWindowBackground(display,bt1,background);
   XSetWindowBackground(display,bt2,background);
   XClearWindow(display,bt1);
   XClearWindow(display,bt2);
   XDrawString(display,bt1,gc,5,12,"Continue",8);
   XDrawString(display,bt2,gc,5,12,"Refresh",7);
}

/***********************************************************
 * unmap and destroy the window and all subwindows;
 * close display connection
 */ 
xp_end_()
{
   XUnloadFont(display, font_info->fid);
   XFreeGC(display, gc);
   XFreePixmap( display, pixmap );
   XDestroyWindow( display, window );
   XFlush( display );
   XCloseDisplay(display);
}     /* x11.c */
